from bot_core.logging.database_logger import DatabaseLogger
from bot_data.abc.migration_abc import MigrationABC
from bot_data.db_context import DBContext


class DBHistoryMigration(MigrationABC):
    name = "1.0.0_DBHistoryMigration"
    prio = 1

    def __init__(self, logger: DatabaseLogger, db: DBContext):
        MigrationABC.__init__(self)
        self._logger = logger
        self._db = db
        self._cursor = db.cursor

    def upgrade(self):
        self._logger.debug(__name__, "Running upgrade")

        self._exec(__file__, "api_keys.sql")
        self._exec(__file__, "auth_users.sql")
        self._exec(__file__, "auth_user_users_relation.sql")
        self._exec(__file__, "auto_role_rules.sql")
        self._exec(__file__, "auto_roles.sql")
        self._exec(__file__, "clients.sql")
        self._exec(__file__, "game_servers.sql")
        self._exec(__file__, "known_users.sql")
        self._exec(__file__, "levels.sql")
        self._exec(__file__, "servers.sql")
        self._exec(__file__, "user_game_idents.sql")
        self._exec(__file__, "user_joined_game_servers.sql")
        self._exec(__file__, "user_joined_servers.sql")
        self._exec(__file__, "user_joined_voice_channel.sql")
        self._exec(__file__, "user_message_count_per_hour.sql")
        self._exec(__file__, "users.sql")
        self._exec(__file__, "user_warnings.sql")

        self._logger.debug(__name__, "Finished history upgrade")

    def downgrade(self):
        self._cursor.execute("DROP TABLE `ApiKeysHistory`;")
        self._cursor.execute("DROP TABLE `AuthUsersHistory`;")
        self._cursor.execute("DROP TABLE `AuthUserUsersRelationsHistory`;")
        self._cursor.execute("DROP TABLE `AutoRoleRulesHistory`;")
        self._cursor.execute("DROP TABLE `AutoRolesHistory`;")
        self._cursor.execute("DROP TABLE `ClientsHistory`;")
        self._cursor.execute("DROP TABLE `GameServersHistory`;")
        self._cursor.execute("DROP TABLE `KnownUsersHistory`;")
        self._cursor.execute("DROP TABLE `LevelsHistory`;")
        self._cursor.execute("DROP TABLE `ServersHistory`;")
        self._cursor.execute("DROP TABLE `UserGameIdentsHistory`;")
        self._cursor.execute("DROP TABLE `UserJoinedGameServerHistory`;")
        self._cursor.execute("DROP TABLE `UserJoinedServersHistory`;")
        self._cursor.execute("DROP TABLE `UserJoinedVoiceChannelHistory`;")
        self._cursor.execute("DROP TABLE `UserMessageCountPerHourHistory`;")
        self._cursor.execute("DROP TABLE `UsersHistory`;")
        self._cursor.execute("DROP TABLE `UserWarningsHistory`;")
