from cpl_query.extension import List

from bot_data.model.client import Client
from bot_graphql.abc.filter_abc import FilterABC


class ClientFilter(FilterABC):
    def __init__(self):
        FilterABC.__init__(self)

        self._id = None
        self._discord_id = None
        self._name = None
        self._server = None

    def from_dict(self, values: dict):
        if "id" in values:
            self._id = int(values["id"])

        if "discordId" in values:
            self._id = int(values["discordId"])

        if "name" in values:
            self._name = values["name"]

        if "server" in values:
            from bot_graphql.filter.server_filter import ServerFilter

            server = ServerFilter()
            server.from_dict(values["server"])
            self._server = server

    def filter(self, query: List[Client]) -> List[Client]:
        if self._id is not None:
            query = query.where(lambda x: x.id == self._id)

        if self._discord_id is not None:
            query = query.where(lambda x: x.id == self._discord_id)

        if self._name is not None:
            query = query.where(lambda x: self._name.lower() == x.name.lower() or self._name.lower() in x.name.lower())

        if self._server is not None:
            servers = self._server.filter(query.select(lambda x: x.server)).select(lambda x: x.id)
            query = query.where(lambda x: x.server.id in servers)

        return query
