from cpl_core.dependency_injection import ServiceProviderABC
from cpl_discord.service import DiscordBotServiceABC
from cpl_query.extension import List

from bot_data.model.user_joined_game_server import UserJoinedGameServer
from bot_graphql.abc.filter_abc import FilterABC


class UserJoinedGameServerFilter(FilterABC):
    def __init__(
        self,
        services: ServiceProviderABC,
        bot: DiscordBotServiceABC,
    ):
        FilterABC.__init__(self)

        self._services = services
        self._bot = bot

        self._id = None
        self._game_server = None
        self._user = None

    def from_dict(self, values: dict):
        if "id" in values:
            self._id = int(values["id"])

        if "gameServer" in values:
            self._game_server = values["gameServer"]

        if "user" in values:
            from bot_graphql.filter.user_filter import UserFilter

            self._user: UserFilter = self._services.get_service(UserFilter)
            self._user.from_dict(values["user"])

    def filter(self, query: List[UserJoinedGameServer]) -> List[UserJoinedGameServer]:
        if self._id is not None:
            query = query.where(lambda x: x.id == self._id)

        if self._game_server is not None:
            query = query.where(lambda x: x.game_server == self._game_server)

        if self._user is not None:
            users = self._user.filter(query.select(lambda x: x.user)).select(lambda x: x.id)
            query = query.where(lambda x: x.user.id in users)

        return query
