from typing import Union

import discord
from cpl_core.configuration import ConfigurationABC
from cpl_core.database.context import DatabaseContextABC
from cpl_core.logging import LoggerABC
from cpl_discord.events import OnReactionAddABC
from cpl_discord.service import DiscordBotServiceABC

from bot_core.configuration.feature_flags_enum import FeatureFlagsEnum
from bot_core.configuration.feature_flags_settings import FeatureFlagsSettings
from bot_data.abc.server_repository_abc import ServerRepositoryABC
from bot_data.abc.user_repository_abc import UserRepositoryABC
from bot_data.model.server_config import ServerConfig
from modules.achievements.achievement_service import AchievementService


class AchievementOnReactionAddEvent(OnReactionAddABC):
    def __init__(
        self,
        config: ConfigurationABC,
        logger: LoggerABC,
        bot: DiscordBotServiceABC,
        achievements: AchievementService,
        db: DatabaseContextABC,
        servers: ServerRepositoryABC,
        users: UserRepositoryABC,
    ):
        OnReactionAddABC.__init__(self)

        self._config = config
        self._logger = logger
        self._bot = bot
        self._achievements = achievements
        self._db = db
        self._servers = servers
        self._users = users

    async def on_reaction_add(
        self,
        reaction: discord.reaction.Reaction,
        user: Union[discord.Member, discord.User],
    ):
        if not isinstance(user, discord.Member):
            return

        if user.guild is None:
            return

        if user.bot:
            return

        server_config: ServerConfig = self._config.get_configuration(f"ServerConfig_{user.guild.id}")
        if not FeatureFlagsSettings.get_flag_from_dict(
            server_config.feature_flags, FeatureFlagsEnum.achievements_module
        ):
            return

        server = self._servers.get_server_by_discord_id(user.guild.id)
        user = self._users.get_user_by_discord_id_and_server_id(user.id, server.id)

        user.reaction_count += 1
        self._db.save_changes()
        self._users.update_user(user)

        await self._achievements.validate_achievements_for_user(user)
