from cpl_core.configuration import ConfigurationABC
from cpl_core.logging import LoggerABC
from cpl_discord.events.on_raw_reaction_remove_abc import OnRawReactionRemoveABC
from cpl_discord.service import DiscordBotServiceABC
from discord import RawReactionActionEvent

from bot_core.configuration.feature_flags_enum import FeatureFlagsEnum
from bot_core.configuration.feature_flags_settings import FeatureFlagsSettings
from bot_core.helper.event_checks import EventChecks
from bot_data.abc.auto_role_repository_abc import AutoRoleRepositoryABC
from bot_data.abc.server_repository_abc import ServerRepositoryABC
from bot_data.model.server_config import ServerConfig
from modules.auto_role.helper.auto_role_reaction_handler import AutoRoleReactionHandler


class AutoRoleOnRawReactionRemoveEvent(OnRawReactionRemoveABC):
    def __init__(
        self,
        config: ConfigurationABC,
        logger: LoggerABC,
        bot: DiscordBotServiceABC,
        servers: ServerRepositoryABC,
        auto_roles: AutoRoleRepositoryABC,
        reaction_handler: AutoRoleReactionHandler,
    ):
        OnRawReactionRemoveABC.__init__(self)

        self._config = config
        self._logger = logger
        self._bot = bot
        self._servers = servers
        self._auto_roles = auto_roles
        self._reaction_handler = reaction_handler

    @EventChecks.check_is_ready()
    async def on_raw_reaction_remove(self, payload: RawReactionActionEvent):
        self._logger.debug(__name__, f"Module {type(self)} started")
        server_config: ServerConfig = self._config.get_configuration(f"ServerConfig_{payload.guild_id}")
        if not FeatureFlagsSettings.get_flag_from_dict(server_config.feature_flags, FeatureFlagsEnum.auto_role_module):
            return

        await self._reaction_handler.handle(payload, "remove")

        self._logger.debug(__name__, f"Module {type(self)} stopped")
